# An implementation for MTT with ACGs

## PART 1: Installing your environment to use these files

### **Download the files**
Choose or create a repository in your file system (for example `MyRepoTST`), and download the files there.

### **Download ACGtk**
see [ACGtk Git repository](https://gitlab.inria.fr/pacg/ACGtk/-/blob/master/INSTALL.md)
Note that we use the 1.5.3-20210712 version of ACGtk here (a new version will soon be released).

### **Compile the files** 
* open a terminal, and go to the repository `MyRepoTST` in which you downloaded the files
* execute the `acgc-files.sh` script, or write the following commands in your terminal :
```
$ acgc deep_syntactic.acg
$ acgc deep_syntactic.acgo semantic.acg
$ acgc deep_syntactic.acgo semantic.acgo dsynt_tree.acg
$ acgc deep_syntactic.acgo semantic.acgo dsynt_tree.acgo dsynt_rule.acg
$ acgc deep_syntactic.acgo semantic.acgo dsynt_tree.acgo dsynt_rule.acgo dsynt_para.acg
$ acgc deep_syntactic.acgo semantic.acgo dsynt_tree.acgo dsynt_rule.acgo dsynt_para.acgo fl.acg
$ acgc deep_syntactic.acgo semantic.acgo dsynt_tree.acgo dsynt_rule.acgo dsynt_para.acgo fl.acgo surface_syntactic.acg
$ acgc deep_syntactic.acgo semantic.acgo dsynt_tree.acgo dsynt_rule.acgo dsynt_para.acgo fl.acgo surface_syntactic.acgo ssynt_tree.acg
```

### **Open the files in ACGtk**
In your terminal, in the `MyRepoTST` repository:
* launch ACGtk :
```
$ acg
```
* In ACGtk, enter the following commands to charge the files:
``` 
load o semantic.acgo;
load o deep_syntactic.acgo;
load o dsynt_tree.acgo;
load o dsynt_para.acgo;
load o dsynt_rule.acgo;
load o fl.acgo;
load o surface_syntactic.acgo;
load o ssynt_tree.acgo;
```
* If you want to see what files are charged, you can use this command (in ACGtk):
```
list;
```
You are now ready to use this repository !

## PART 2: Using our MTT implementation

### **Basic commands syntax**
* to apply a lexicon `myLex` over a given formula `myFormula` where `myType` is the type of `myFormula` in the abstract signature of `myLex`:
```
myLex realize myFormula : myType;
``` 
* to reverse a lexicon `myLex` over a given formula `myFormula` where `myType` is the type of `myFormula` in the abstract signature of `myLex`:
```
myLex parse myFormula : myType;
``` 

### **Commands pattern in the direction of generation**
Note that in the following examples, the "`...`" are to be replaced with the adapted formula or structure. 
* From SemS to DSyntR:
```
sem realize ... : G;  # ... : formula composed of the deep-syntactic signature terms
```
**Remark** : the above command will return the semantic graph linked to the given formula. In fact, to start with constants from the `semantic` signature, the command would have been `sem parse ... : G;` (where `...` is a formula composed of terms from the `semantic` signature), but due to some type related concerns (see section Remarks below), we give here the reverse command instead.
```
dsyntRel realize ... : G;  #  ... : formula composed of the deep-syntactic signature terms
```
* If you want to make some lexical function appear:
```
lexfl parse ... : t;  # ... : formula composed of the dsynt-tree signature terms
reducefl realize ... : t;  # ... : formula composed of the fl signature terms
reducefl parse ... : t;  # ... : formula composed of the dsynt-0-fl signature terms
lexfl realize ... : t;  # ... : formula composed of the fl signature terms
```
* To perform the deep-syntactic paraphrase:
```
lexpara parse ... : T;  # ... : formula composed of the dsynt-tree signature terms
pararule realize ... : T;  # ... : formula composed of the dsynt-para signature terms
pararule parse ... : T;  # ... : formula composed of the dsynt-rule signature terms
lexpara realize ... : T;  # ... : formula composed of the dsynt-para signature terms
```
* To find some other choices of lexical functions applications:
```
lexfl parse ... : t;  # ... : formula composed of the dsynt-tree signature terms
reducefl realize ... : t;  # ... : formula composed of the fl signature terms
reducefl parse ... : t;  # ... : formula composed of the dsynt-0-fl signature terms
lexfl realize ... : t;  # ... : formula composed of the fl signature terms
```
* To realize lexical functions:
```
lexfl parse ... : t;  # ... : formula composed of the dsynt-tree signature terms
reducefl realize ... : t;  # ... : formula composed of the fl signature terms
```
* From a deep-syntactic structure without lexical functions to SSyntS:
```
ss2dt parse ... : S;  # ... : formula composed of the dsynt-0-fl signature terms
ssyntRel realize ... : T;  # ... : formula composed of the surface-syntactic signature terms
```
Note that instead of `S` in this last block of commands, you can also use `NP`, `AdvP` or `AdjP` depending on what kind of expression you are handling.

### **Some example sentences (in signature `deep-syntactic`)**
Here are examples of some representation in the `deep_syntactic` signature for the following expressions:
1. *John murders Mary.*
```
ds_murder (EXP ds_john) ds_mary
```
2. *Alain is murderd.*
```
ds_murder IMP ds_alain
```
3. *Charlie kills Taylor.*
```
ds_kill ds_I_MOD (EXP ds_charlie) ds_taylor
```
4. *Charlie is killed.*
```
ds_kill ds_I_MOD IMP ds_charlie
```
5. *Mary muders John for a political reason.*
```
(ds_for_pol_reas (lambda A x. ds_murder A x ds_john)) (EXP ds_mary)
```
6. *Taylor is murdered for a political reason.*
```
(ds_for_pol_reas (lambda A x. ds_murder A x ds_taylor)) (IMP)
```
7. *Taylor kills John intentionnally.*
```
(ds_intentionally (lambda A x. ds_kill A x ds_john)) (EXP ds_taylor)
```
8. *Mary the wife of John* or *Mary, John's wife*
```
ds_wife (EXP ds_mary) ds_john
```
9. *Alain's wife*
```
ds_wife IMP ds_alain
```
10. *Mary, the daughter in law of John*
```
ds_daughter_in_law_0 (EXP ds_mary) ds_john
ds_daughter_in_law_1 (EXP ds_mary) ds_john
```
11. *Alain is calm.*
```
ds_be ds_alain ds_calm
```
12. *Alain is upset.*
```
ds_be ds_alain ds_upset
```

### **Remarks**
Due to the handling of adverbial groups, the parsing of both lexicons `sem` and `dsyntRel` are not yet possible. Indeed the constants representing them in the abstract signature `deep-syntactic` have higher order types. In order to enable the parsing operation for these lexicons, you need to comment the lines corresponding to the constants `ds_for_pol_reas`, `ds_for_spec_reas` and `ds_intentionally` in the signature `deep_syntactic` (file `deep_syntactic.acg`), in the lexicon `sem` (file `semantic.acg`), and in the lexicon `dsyntRel` (file `dsynt_tree.acg`), as well as replace, in the file `semantic.acg` all the use of the constants `pred0`, `pred1`, etc. by their expression.

Moreover, we dont have the logical equivalence between *A & B* and *B & A* in our implementation, for it uses lambda calculus. Therefore, some expression may share the same (logically speaking) representation in some signature, but their representation may differ in the order of the subexpressions. That is due to the way they were implemented. 


### ***Charlie murders Taylor* and *Charlie kills Taylor intentionally***
Both sentences (1) and (7) have the same semantic representation, but lead to two different surface realizations.

Here are the commands to generate a surface syntactic realization of *Charlie murders Taylor* (1):
```
# sem parse Lambda e0. 
#       Ex ex. Ex ey. 
#       (((((s_kill e0) & (R1 e0 ex)) & (R2 e0 ey)) & (s_charlie ex)) & (s_taylor ey)) &
#       (Ex e1. ((s_intentionally e1) & (R1 e1 ex)) & (R2 e1 e0)) 
#       : n => t;
# should return: ds_murder (EXP ds_charlie) ds_taylor : G
# you can verify it with this command instead :
sem realize ds_murder (EXP ds_charlie) ds_taylor : G;

dsyntRel realize ds_murder (EXP ds_charlie) ds_taylor : G;
# returns: lex2 dt_MURDER A1 (lex0 dt_CHARLIE) A2 (lex0 dt_TAYLOR) : T

lexfl parse lex2 dt_MURDER A1 (lex0 dt_CHARLIE) A2 (lex0 dt_TAYLOR) : t;
# returns: lex2 fl_MURDER a1 (lex0 fl_CHARLIE) a2 (lex0 fl_TAYLOR) : t

reducefl realize lex2 fl_MURDER a1 (lex0 fl_CHARLIE) a2 (lex0 fl_TAYLOR) : t;
# returns: lex2 d0f_MURDER a1 (lex0 d0f_CHARLIE) a2 (lex0 d0f_TAYLOR) : t

ss2ds parse lex2 d0f_MURDER a1 (lex0 d0f_CHARLIE) a2 (lex0 d0f_TAYLOR) : S;
# returns: ss_MURDER12 ss_CHARLIE0 ss_TAYLOR0 : S

ssyntRel realize ss_MURDER12 ss_CHARLIE0 ss_TAYLOR0 : S;
# returns: verb2 st_MURDER Subj (noun0 st_CHARLIE) DirO (noun0 st_TAYLOR) : T
```

Here are the commands to generate a surface syntactic realization of *Charlie kills Taylor intentionally* (7):
```
# sem parse Lambda e0. 
#       Ex e1. ((s_intentionally e1) & (R2 e1 e0)) & 
#       (Ex ex. Ex ey. (((((s_kill e0) & (R1 e0 ex)) & (R2 e0 ey)) & true) & 
#       ((R1 e1 ex) & (s_charlie ex))) & (s_taylor ey)) 
#       : n => t;
# should return: (ds_intentionally (lambda A x. ds_kill A x ds_taylor)) (EXP ds_charlie) : G
# you can verify it with this command instead :
sem realize (ds_intentionally (lambda A x. ds_kill A x ds_taylor)) (EXP ds_charlie) : G;

dsyntRel realize (ds_intentionally (lambda A x. ds_kill A x ds_taylor)) (EXP ds_charlie) : G;
# returns: lex3 dt_KILL A1 (lex0 dt_CHARLIE) A2 (lex0 dt_TAYLOR) ATTR (lex0 dt_INTENTIONALLY) : T

lexfl parse lex3 dt_KILL A1 (lex0 dt_CHARLIE) A2 (lex0 dt_TAYLOR) ATTR (lex0 dt_INTENTIONALLY) : t;
# returns: lex3 fl_KILL a1 (lex0 fl_CHARLIE) a2 (lex0 fl_TAYLOR) attr (lex0 fl_INTENTIONALLY) : t

reducefl realize lex3 fl_KILL a1 (lex0 fl_CHARLIE) a2 (lex0 fl_TAYLOR) attr (lex0 fl_INTENTIONALLY) : t;
# returns: lex3 d0f_KILL a1 (lex0 d0f_CHARLIE) a2 (lex0 d0f_TAYLOR) attr (lex0 d0f_INTENTIONALLY) : t

ss2ds parse lex3 d0f_KILL a1 (lex0 d0f_CHARLIE) a2 (lex0 d0f_TAYLOR) attr (lex0 d0f_INTENTIONALLY) : S;
# returns: ss_KILL12A ss_CHARLIE0 ss_TAYLOR0 ss_INTENTIONALLY0 : S

ssyntRel realize ss_KILL12A ss_CHARLIE0 ss_TAYLOR0 ss_INTENTIONALLY0 : S;
# returns: verb2adv st_KILL Subj (noun0 st_CHARLIE) DirO (noun0 st_TAYLOR) (gpe_adv Circum (adv0 st_INTENTIONALLY)) : T;
```

### **Some interesting tests and results**
Expressions (1) and (7) have the same semantic representation (provided we use the same names in them):
```
sem realize ds_murder (EXP ds_charlie) ds_taylor : G;
sem realize (ds_intentionally (lambda A x. ds_kill A x ds_taylor)) (EXP ds_charlie) : G;
```

Second expression (10) and *Mary, the wife of John's son* have the same semantic representation. Both following commands give the same semantic graph:
```
sem realize ds_daughter_in_law_1 (EXP ds_mary) ds_john : G;
sem realize ds_wife (EXP ds_mary) (ds_son IMP ds_john) : G;
``` 
And if you parse this graph you obtain the two above expressions (you need to comment the lines encoding the adverbial groups in the signatures and lexicon you want to use first, here in the semantic signature, sem lexicon, and deep-syntactic signature).
```
sem parse Lambda e0.
    Ex ex. Ex e1.
    ((((s_wife e0) & (R1 e0 ex)) & (R2 e0 e1)) & (s_mary ex)) &
    (Ex e2. Ex ey. ((((s_son e1) & (R1 e1 e2)) & (R2 e1 ey)) & 
    (Ex_v v. v e2)) & (s_john ey))
    : G;
```
Both expressions (10) have the same deep-syntactic representation. The following command will give you the two of them.
```
dsyntRel parse lex2 dt_DAUGHTER_IN_LAW A1 (lex0 dt_MARY) A2 (lex0 dt_JOHN) : G;
```